# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA


class EntryType(object):
	def __init__(self, name):
		self._name = name
		self._required_fields = []
		self._optional_fields = []
	
	@property
	def name(self):
		return self._name

	@property
	def required_fields(self):
		return self._required_fields
	
	@property
	def optional_fields(self):
		return self._optional_fields


class Field(object):
	def __init__(self, name, label):
		self._name = name
		self._label = label
	
	@property
	def name(self):
		return self._name
	
	@property
	def label(self):
		return self._label


class BibTeXDefinition(object):
	"""
	Holds a definition for BibTeX
	"""
	def __init__(self):
		self._fields = {}
		self._types = []
	
	@property
	def fields(self):
		return self._fields
	
	@property
	def types(self):
		return self._types


from xml.sax import ContentHandler, parse


class BibTeXDefinitionParser(ContentHandler):
	S_IDLE, S_FIELDS, S_REQUIRED, S_OPTIONAL = 0, 1, 2, 3
	
	def __init__(self):
		self._state = self.S_IDLE
		self._type = None
	
	def startElement(self, name, attributes):
		if name == "fields":
			self._state = self.S_FIELDS		# we are now parsing the known fields
		elif name == "required":
			self._state = self.S_REQUIRED
		elif name == "optional":
			self._state = self.S_OPTIONAL
		elif name == "field":
			if self._state == self.S_FIELDS:
				# we are in <fields>
				id = attributes["id"]
				self._definition.fields[id] = Field(id, attributes["_label"])
			elif self._state == self.S_REQUIRED:
				# we are in <types><type><required>
				id = attributes["id"]
				try:
					f = self._definition.fields[id]
					self._type.required_fields.append(f)
				except KeyError:
					raise Exception("Field '%s' not declared in <fields>" % id)
			elif self._state == self.S_OPTIONAL:
				# we are in <types><type><optional>
				id = attributes["id"]
				try:
					f = self._definition.fields[id]
					self._type.optional_fields.append(f)
				except KeyError:
					raise Exception("Field '%s' not declared in <fields>" % id)
			else:
				raise Exception("<field> is not allowed here")
		elif name == "type":
			t = EntryType(attributes["id"])
			self._definition.types.append(t)
			self._type = t
			
	def parse(self, definition, filename):
		self._definition = definition
		parse(filename, self)


from gtk import Dialog, Notebook, Label, Entry, DIALOG_MODAL, DIALOG_DESTROY_WITH_PARENT, STOCK_CANCEL, \
		RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK, combo_box_new_text, Table, VBox, FILL, TextView, TextBuffer, \
		ScrolledWindow, POLICY_NEVER, POLICY_AUTOMATIC, SHADOW_IN

from Installation import Installation


class BibTeXEntryDialog(Dialog):
	def __init__(self, parent):
		Dialog.__init__(self, "Insert BibTeX Entry", parent, DIALOG_MODAL|DIALOG_DESTROY_WITH_PARENT, 
					    (STOCK_CANCEL, RESPONSE_CANCEL, STOCK_OK, RESPONSE_OK))
		self.set_border_width(5)
		self.set_has_separator(False)
		self.vbox.set_spacing(10)
		
		# load definition
		
		self._definition = BibTeXDefinition()
		parser = BibTeXDefinitionParser()
		parser.parse(self._definition, "%s/bibtex.xml" % Installation().path)
		
		self._mapping = {}	# maps Entry objects to field names
		
		# build ui
		
		lbl_type = Label("_Type:")
		lbl_type.set_use_underline(True)
		lbl_type.set_alignment(0, .5)
		
		self._cmb_type = combo_box_new_text()
		for t in self._definition.types:
			self._cmb_type.append_text(t.name)
		self._cmb_type.connect("changed", self._type_changed)
		
		lbl_ident = Label("_Identifier:")
		lbl_ident.set_use_underline(True)
		lbl_ident.set_alignment(0, .5)
		
		self._ent_ident = Entry()
		
		# pages
		
		self._page_required = VBox()
		self._page_required.set_border_width(5)
		self._page_optional = VBox()
		self._page_optional.set_border_width(5)
		
		self._view_abstract = TextView(TextBuffer())
		self._scr_abstract = ScrolledWindow()
		self._scr_abstract.set_policy(POLICY_NEVER, POLICY_AUTOMATIC);
		self._scr_abstract.set_shadow_type(SHADOW_IN)
		self._scr_abstract.set_border_width(5)
		self._scr_abstract.add(self._view_abstract)
		
		# pack
		
		notebook = Notebook()
		notebook.append_page(self._page_required, Label("Required Fields"))
		notebook.append_page(self._page_optional, Label("Optional Fields"))
		notebook.append_page(self._scr_abstract, Label("Abstract"))
		
		table = Table()
		table.set_border_width(5)
		table.set_row_spacings(5)
		table.set_col_spacings(5)
		table.attach(lbl_type, 0, 1, 0, 1, xoptions=FILL, yoptions=0)
		table.attach(self._cmb_type, 1, 2, 0, 1, yoptions=0)
		table.attach(lbl_ident, 2, 3, 0, 1, xoptions=FILL, yoptions=0)
		table.attach(self._ent_ident, 3, 4, 0, 1, yoptions=0)
		table.attach(notebook, 0, 4, 1, 2)
		table.set_col_spacing(1, 15)
		
		table.show_all()
		
		self.vbox.add(table)
		
		self._cmb_type.set_active(0)
		
	def _type_changed(self, cmb):
		"""
		Another entry type has been selected so we produce the according labels and entries in
		the required and optional page
		"""
		i = self._cmb_type.get_active()
		type = self._definition.types[i]
		
		self._mapping = {}
		
		# required fields
		
		tbl_required = Table()
		tbl_required.set_border_width(5)
		tbl_required.set_row_spacings(5)
		tbl_required.set_col_spacings(5)
		i = 0
		
		for field in type.required_fields:
			label = Label(field.label + ":")
			label.set_alignment(0, .5)
			
			entry = Entry()
			
			tbl_required.attach(label, 0, 1, i, i + 1, xoptions=FILL)
			tbl_required.attach(entry, 1, 2, i, i + 1)
			
			self._mapping[entry] = field.name
			
			i += 1
		
		for child in self._page_required.get_children():
			child.destroy()
		
		tbl_required.show_all()
		self._page_required.pack_start(tbl_required, False)
		
		# optional fields
		
		tbl_optional = Table()
		tbl_optional.set_border_width(5)
		tbl_optional.set_row_spacings(5)
		tbl_optional.set_col_spacings(5)
		i = 0
		
		for field in type.optional_fields:
			label = Label(field.label + ":")
			label.set_alignment(0, .5)
			
			entry = Entry()
			
			tbl_optional.attach(label, 0, 1, i, i + 1, xoptions=FILL)
			tbl_optional.attach(entry, 1, 2, i, i + 1)
			
			self._mapping[entry] = field.name
			
			i += 1
		
		for child in self._page_optional.get_children():
			child.destroy()
		
		tbl_optional.show_all()
		self._page_optional.pack_start(tbl_optional, False)
	
	@property
	def source(self):
		"""
		Generate a BibTeX entry from the dialog
		"""
		s = "@%s{%s,\n" % (self._cmb_type.get_active_text(), self._ent_ident.get_text())
		
		# fields
		f = []
		for entry, field in self._mapping.iteritems():
			value = entry.get_text()
			if len(value):
				f.append("\t%s = {%s}" % (field, value))
		s += ",\n".join(f) 
		
		# abstract
		buf = self._view_abstract.get_buffer()
		abs = buf.get_text(buf.get_start_iter(), buf.get_end_iter())
		if len(abs):
			s += ",\n\tabstract = {%s}" % abs
		
		s += "\n}"
		
		return s
	

if __name__ == "__main__":
	d = BibTeXEntryDialog(None)
	d.run()
	import gtk
	gtk.main()
		
		