# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import ComboBox, ListStore, CellRendererText
from gtk.gdk import screen_width, screen_height, screen_width_mm, screen_height_mm

from Logger import debug, error

# unit constants
PIXEL, INCH, MILLIMETER, CENTIMETER, POINT, PERCENT = range(6)

class UnitConverter(object):
	"""
	This is used to convert values from one unit to another
	"""
	def __init__(self):
		self.__guess_dpi()
		
		self.factors = [[0, 0, 0, 0, 0], [0, 0, 0, 0, 0], [0, 0, 0, 0, 0], [0, 0, 0, 0, 0], [0, 0, 0, 0, 0]]
		
		self.factors[PIXEL][INCH] = 1.0 / self.__dpi				# 1 px = 1/DPI in
		self.factors[PIXEL][MILLIMETER] = 25.4 / self.__dpi		# 1 in = 25.4 mm
		self.factors[PIXEL][CENTIMETER] = 2.54 / self.__dpi
		self.factors[PIXEL][POINT] = 72.0 / self.__dpi			# 1 pt = 1/72 in
		
		self.factors[INCH][PIXEL] = self.__dpi
		self.factors[INCH][MILLIMETER] = 25.4
		self.factors[INCH][CENTIMETER] = 2.54
		self.factors[INCH][POINT] = 72.0
		
		self.factors[MILLIMETER][PIXEL] = self.__dpi / 25.4
		self.factors[MILLIMETER][INCH] = 1.0 / 25.4
		self.factors[MILLIMETER][CENTIMETER] = 0.1
		self.factors[MILLIMETER][POINT] = 72.0 / 25.4
		
		self.factors[CENTIMETER][PIXEL] = self.__dpi / 2.54
		self.factors[CENTIMETER][INCH] = 1.0 / 2.54
		self.factors[CENTIMETER][MILLIMETER] = 10.0
		self.factors[CENTIMETER][POINT] = 72.0 / 2.54
		
		self.factors[POINT][PIXEL] = self.__dpi / 72.0
		self.factors[POINT][INCH] = 1.0 / 72.0
		self.factors[POINT][MILLIMETER] = 25.4 / 72.0
		self.factors[POINT][CENTIMETER] = 2.54 / 72.0
	
	def __guess_dpi(self):
		"""
		Try to guess the DPI of the screen
		"""
		dpi_x = screen_width() / (screen_width_mm() / 25.4)
		dpi_y = screen_height() / (screen_height_mm() / 25.4)
		
		self.__dpi = (dpi_x + dpi_y) / 2.0
		
		debug(self, "__guess_dpi", str(self.__dpi))
	
	def convert(self, value, from_unit, to_unit):
		if from_unit == to_unit:
			return value
		else:
			return value * self.factors[from_unit][to_unit]


units = { PIXEL : "px", INCH : "in", MILLIMETER : "mm", CENTIMETER : "cm", POINT : "pt" }

class UnitComboBox(ComboBox):
	def __init__(self, default_unit=PIXEL):
		self.__model = ListStore(str, int)
		ComboBox.__init__(self, self.__model)
		
		i = 0
		default_unit_index = 0
		for unit, label in units.iteritems():
			if unit == default_unit:
				default_unit_index = i
			self.__model.append([label, unit])
			i += 1
		
		renderer = CellRendererText()
  		self.pack_start(renderer, True)
  		self.add_attribute(renderer, "text", 0)
  		
  		self.__spinbuttons = []
  		self.__unit = default_unit
  		self.__monitor_spinbuttons = True
  		self.__keep_aspect_ratio = False
  		self.__enable_percent = False
  		self.__initials = {}		# maps SpinButton objects to their initial values and units (used for %)
  		
  		self.__converter = UnitConverter()
  		
  		self.set_active(default_unit_index)
  		
  		self.connect("changed", self.__changed)
		
	def attach_spinbutton(self, spinbutton):
		spinbutton.connect("changed", self.__spinbutton_changed)
		self.__spinbuttons.append(spinbutton)
		self.__initials[spinbutton] = (0, PIXEL)
		
		#log(self, str(self.__spinbuttons))
	
	def set_initial(self, spinbutton, value, unit):
		self.__initials[spinbutton] = (value, unit)
		
		#log(self, str(self.__initials))
	
	def __spinbutton_changed(self, spinbutton):
		if not self.__keep_aspect_ratio:
			return
		
		if self.__monitor_spinbuttons:
			self.__monitor_spinbuttons = False	# avoid loops
			
			#
			# find other spinbutton
			#
			other_spinbutton = None
			for s in self.__spinbuttons:
				if s is not spinbutton:
					other_spinbutton = s
					break
			if not other_spinbutton:
				log(self, "__spinbutton_changed : could not find other spin button")
				self.__monitor_spinbuttons = True
				return
			
			#
			# set value
			#
			if self.__unit == PERCENT:
				other_spinbutton.set_value(spinbutton.get_value())
			else:
				ratio = float(self.__initials[spinbutton][0]) / float(self.__initials[other_spinbutton][0])
				other_spinbutton.set_value(float(spinbutton.get_value()) / ratio)
				
			self.__monitor_spinbuttons = True

	def __changed(self, combobox):
		unit = self.__model[self.get_active()][1]
		
		self.__monitor_spinbuttons = False
		
		for spinbutton in self.__spinbuttons:
			if self.__unit == PERCENT:
				# FROM percent...
				p = spinbutton.get_value()						# percents of initial [initial]
				i_value_i = self.__initials[spinbutton][0]		# initial [initial]
				c_value_i = (float(i_value_i) * float(p)) / 100.0	# current [initial]
				i_unit =  self.__initials[spinbutton][1]
				n_value = self.__converter.convert(c_value_i, i_unit, unit)
				
			elif unit == PERCENT:
				# TO percent...
				# -> convert from current unit to initial unit and then to percent
				
				c_value_c = spinbutton.get_value()			# current [current]
				i_unit = self.__initials[spinbutton][1]
				c_value_i = self.__converter.convert(c_value_c, self.__unit, i_unit)	# current [initial]
				i_value_i = self.__initials[spinbutton][0]	# initial [initial]
				try:
					n_value = float(c_value_i) / float(i_value_i) * 100
				except ZeroDivisionError:
					#
					# TODO: this happens after init, but it shouldn't...
					#
					debug(self, "__changed", "ZeroDivisionError")
					n_value = 0
			else:
				n_value = self.__converter.convert(spinbutton.get_value(), self.__unit, unit)
			
			spinbutton.set_value(n_value)
			
			# set digits
			if unit == PIXEL:
				spinbutton.set_digits(0)
			else:
				spinbutton.set_digits(2)
		
		self.__unit = unit
		self.__monitor_spinbuttons = True
	
	def set_enable_percent(self, enable_percent):
		if self.__enable_percent and not enable_percent:
			# switch off
			#
			# TODO: implement
			#
			raise NotImplementedError
		elif not self.__enable_percent and enable_percent:
			# switch on
			self.__model.append([r"%", PERCENT])
			
		self.__enable_percent = enable_percent
	
	def set_unit(self, unit):
		# hm ...
		i = 0
		for row in self.__model:
			if row[1] == unit:
				self.set_active(i)
				return
			i += 1
	
	def get_unit(self):
		return self.__model[self.get_active()][1]
	
	def get_unit_string(self):
		u = self.get_unit()
		return { PIXEL : "px", INCH : "in", MILLIMETER : "mm", CENTIMETER : "cm", POINT : "pt", PERCENT : r"%" }[u]
	
	def set_keep_aspect_ratio(self, keep_aspect_ratio):
		if len(self.__spinbuttons) > 2:
			error(self, "set_keep_aspect_ratio", "keep_aspect_ratio enabled for > 2 attached spin buttons")
		
		self.__keep_aspect_ratio = keep_aspect_ratio

