# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gobject import GObject, type_register, SIGNAL_RUN_LAST, TYPE_NONE, TYPE_INT, TYPE_PYOBJECT
from gedit import tab_get_from_document

from LaTeXPlugin.Logger import debug
from DocumentHelper import DocumentHelper
from LaTeXPlugin.bibtex.BibTeXModelFactory import BibTeXModelFactory

class DocumentHelperManager(GObject):
	__gsignals__ = {
		"active-language-changed" : (SIGNAL_RUN_LAST, TYPE_NONE, ([TYPE_INT])),
		"active-helper-changed" : (SIGNAL_RUN_LAST, TYPE_NONE, ([TYPE_PYOBJECT]))
	}
	
	def __init__(self, window_helper):
		GObject.__init__(self)
		
		self.__window_helper = window_helper
		self.__window = window_helper.window
		
		self.__handler_id = None
		
		self.__bibtex_model_factory = BibTeXModelFactory()
		
		# initialize helpers
		self.__document_helpers = {}
		self.__active_document_helper = None
		active_view = self.__window.get_active_view()
		for view in self.__window.get_views():
			helper = DocumentHelper(self, view)
			self.__document_helpers[view] = helper
			if view == active_view:
				self.__active_document_helper = helper
				self.__handler_id = helper.connect("language-changed", self.__active_language_changed)
		
		# connect to window signals
		self.__window.connect("tab_added", self.__tab_added)
		self.__window.connect("tab_removed", self.__tab_removed)
		self.__window.connect("active_tab_changed", self.__active_tab_changed)
	
	def __tab_added(self, window, tab):
		debug(self, "__tab_added")
		
		view = tab.get_view()
		if not view in self.__document_helpers:
			self.__document_helpers[view] = DocumentHelper(self, view)
	
	def __tab_removed(self, window, tab):
		view = tab.get_view()
		del self.__document_helpers[view]
	
	def __active_tab_changed(self, window, tab):
		debug(self, "__active_tab_changed")
		
		# disconnect the current's helper "features-changed" signal
		if self.__handler_id:
			self.__active_document_helper.disconnect(self.__handler_id)
		
		# eventually add a new DocumentHelper
		view = tab.get_view()
		try:
			document_helper = self.__document_helpers[view]
		except KeyError:
			document_helper = DocumentHelper(self, view)
			self.__document_helpers[view] = document_helper
		
		# set active helper
		self.__active_document_helper = document_helper
		
		# listen to changing features
		self.__handler_id = self.__active_document_helper.connect("language-changed", 
																self.__active_language_changed)
		
		self.emit("active-helper-changed", self.__active_document_helper)
	
	def activate_helper(self, filename):
		helper = self.find_helper(filename)
		tab = tab_get_from_document(helper.view.get_buffer())
		self.__window.set_active_tab(tab)
		return helper
	
	def __active_language_changed(self, document_helper, language):
		debug(self, "__active_language_changed")
		self.emit("active-language-changed", language)
	
	@property
	def active_helper(self):
		return self.__active_document_helper
	
	def find_helper(self, filename):
		for helper in self.__document_helpers.itervalues():
			if helper.filename == filename:
				return helper
		raise KeyError
	
	@property
	def window_helper(self):
		return self.__window_helper
	
	@property
	def bibtex_model_factory(self):
		return self.__bibtex_model_factory
		
type_register(DocumentHelperManager)
