# -*- coding: utf-8 -*-

# This file is part of the Gedit LaTeX Plugin
#
# Copyright (C) 2007 Michael Zeising
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public Licence as published by the Free Software
# Foundation; either version 2 of the Licence, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public Licence for more 
# details.
#
# You should have received a copy of the GNU General Public Licence along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA  02110-1301, USA

from gtk import Toolbar, ToolButton, Image, Tooltips, SeparatorToolItem, MenuToolButton, RESPONSE_OK, Menu, ImageMenuItem, SeparatorMenuItem, \
	STOCK_PREFERENCES, image_new_from_stock, ICON_SIZE_SMALL_TOOLBAR, TOOLBAR_BOTH_HORIZ
	
from LaTeXPlugin.common import ConditionalSource, TEXT, MATH
from LaTeXPlugin.Installation import Installation
from LaTeXPlugin.Logger import debug
from LaTeXPlugin.util import str_bool
from LaTeXPlugin.document.DocumentHelper import LANGUAGE_NONE, LANGUAGE_LATEX, LANGUAGE_BIBTEX
from LaTeXPlugin.builder.ImageProfileGenerator import ImageProfileGenerator


class LaTeXToolbar(Toolbar):
	def __init__(self, window_helper):
		self.__window_helper = window_helper
		
		Toolbar.__init__(self)
		
		# build on expose event
		self.__built = False
		self.__handler_expose = self.connect("expose-event", self.__exposed)
	
	def __exposed(self, widget, event):
		self.disconnect(self.__handler_expose)
		
		self.set_style(TOOLBAR_BOTH_HORIZ)
		
		self.tooltips = Tooltips()
		
		## quickstart
		self.__insert_tool_button("quickstart", _("New LaTeX"), _("Create a new LaTeX document"), lambda x: self.__window_helper.run_quickstart_dialog(), True)
		
		self.insert(SeparatorToolItem(), -1)
		
		## font style commands
		mnu_font_style = FontStyleMenu(self.__window_helper)
		btn_font_style = SmartMenuToolButton(_("Style"), mnu_font_style)
		btn_font_style.set_tooltip(self.tooltips, _("Change style or typeface"))
		self.insert(btn_font_style, -1)
		
		## size
		mnu_font_size = FontSizeMenu(self.__window_helper)
		btn_font_size = SmartMenuToolButton(_("Font Size"), mnu_font_size)
		btn_font_size.set_tooltip(self.tooltips, _("Change font size"))
		self.insert(btn_font_size, -1)
		
		## justify commands
		mnu_justify = JustifyMenu(self.__window_helper)
		btn_justify = SmartMenuToolButton(_("Justify"), mnu_justify)
		btn_justify.set_tooltip(self.tooltips, _("Change justification"))
		self.insert(btn_justify, -1)
		
		self.insert(SeparatorToolItem(), -1)
		
		## list commands
		self.__insert_format_button("itemize", _("Itemize"), _("Create new item list\n\t⇨\t\\begin{itemize}"), {TEXT : ["\\begin{itemize}\n\t\\item ", "\n\\end{itemize}"]})
		self.__insert_format_button("enumerate", _("Enumerate"), _("Create new enumeration list\n\t⇨\t\\begin{enumerate}"), {TEXT : ["\\begin{enumerate}\n\t\\item ", "\n\\end{enumerate}"]})
		self.__insert_format_button("description", _("Description"), _("Create new description list\n\t⇨\t\\begin{description}"),  {TEXT : ["\\begin{description}\n\t\\item[", "]\n\\end{description}"]})
		self.__insert_format_button("item", _("Item"), _(u"Insert a list item\n\t⇨\t\\item"),  {TEXT : "\\item "})
		
		self.insert(SeparatorToolItem(), -1)
		
		## structure commands
		mnu_structure = StructureMenu(self.__window_helper)
		btn_structure = SmartMenuToolButton(_("Structure"), mnu_structure, 3)
		btn_structure.set_tooltip(self.tooltips, _("Insert structural command"))
		self.insert(btn_structure, -1)
		
		self.insert(SeparatorToolItem(), -1)
		
		## math environments
		mnu_math = MathMenu(self.__window_helper)
		btn_math = SmartMenuToolButton(_("Math"), mnu_math)
		btn_math.set_tooltip(self.tooltips, _("Insert mathematical environment"))
		self.insert(btn_math, -1)
		
		## sub-/superscript, fraction
#		self.__insert_format_button("sub", _("Subscript"), _("Insert subscript text\n\t⇨\t_{}"), {MATH : ["_{", "}"]})
#		self.__insert_format_button("super", _("Superscript"), _("Insert superscript text\n\t⇨\t^{}"), {MATH : ["^{", "}"]})
#		self.__insert_format_button("frac", _("Fraction"), _("Insert fraction\n\t⇨\t\\frac"), {MATH : ["\\frac{", "}{}"]})
		
		## accents
		mnu_accent = AccentMenu(self.__window_helper)
		btn_accent = SmartMenuToolButton(_("Accent"), mnu_accent)
		btn_accent.set_tooltip(self.tooltips, _("Insert an accent"))
		self.insert(btn_accent, -1)
		
		self.insert(SeparatorToolItem(), -1)
		
		## table
		self.__insert_tool_button("table", _("Table"), _("Insert a tabular object"), lambda x: self.__window_helper.run_table_dialog())
		
		## graphics
		self.__insert_tool_button("graphics", _("Graphics"), _("Insert a graphics"), lambda x: self.__window_helper.run_graphics_dialog())
		
		# listing
		self.__insert_tool_button("listing", _("Listing"), _("Insert a source code listing"), lambda x: self.__window_helper.listing_controller.run_dialog())
		
		self.insert(SeparatorToolItem(), -1)
		
		## build
		mnu_build = BuildMenu(self.__window_helper)
		btn_build = SmartMenuToolButton(_("Build"), mnu_build)
		btn_build.set_tooltip(self.tooltips, _("Build a document from the LaTeX source"))
		self.insert(btn_build, -1)
		
		# build image
		self.__insert_tool_button("compile-image", _("Build Image"), _("Build an image from the LaTeX source"), lambda x: self.__build_image())
		
		self.show_all()
		
		self.__built = True
	
	def __insert_format_button(self, icon, label, tooltip, source):
		self.__insert_tool_button(icon, label, tooltip, 
				lambda x: self.__window_helper.document_helper_manager.active_helper.language_helper.insert_source(source))
	
	def __insert_tool_button(self, icon, label, tooltip, callback, is_important=False):
		image = Image()
		image.set_from_file("%s/pixmaps/%s.png" % (Installation().path, icon))
		button = ToolButton(image, label)
		button.set_tooltip(self.tooltips, tooltip)
		button.set_is_important(is_important)
		if callback:
			button.connect("clicked", callback)
		self.insert(button, -1)
	
	def __build_image(self):
		generator = ImageProfileGenerator()
		profile = generator.run()
		if profile:
			self.__window_helper.build(profile)
	
	def set_language(self, language):
		if not self.__built:
			return
		
		if language == LANGUAGE_LATEX:
			for child in self.get_children():
				child.set_sensitive(True)
		else:
			for child in self.get_children():
				child.set_sensitive(False)
			self.get_nth_item(0).set_sensitive(True)
		
class BuildMenu (Menu):
	def __init__(self, window_helper):
		"""
		Load profiles
		"""
		Menu.__init__(self)
		self.__window_helper = window_helper
		self.__items = {}
		
		for profile in window_helper.settings.list_profiles():
			self.__append_profile_item(profile)
		
		window_helper.settings.connect("profile-saved", self.__profile_saved)
		window_helper.settings.connect("profile-deleted", self.__profile_deleted)
	
	def __append_profile_item(self, profile):
		image = Image()
		
		#debug(self, "__append_profile_item", "Icon: " + profile.icon)
		
		image.set_from_file(profile.icon)
		item = ImageMenuItem(profile.name)
		item.set_image(image)
		item.show()
		item.connect("activate", self.__profile_activated, profile.name)
		
		self.__items[profile.name] = item
		self.append(item)
	
	def __profile_saved(self, settings, profile):
		self.__append_profile_item(profile)
	
	def __profile_deleted(self, settings, profile):
		item = self.__items[profile.name]
		self.remove(self.__items[profile.name])
	
	def __profile_activated(self, item, name):
		profile = self.__window_helper.settings.get_profile(name)
		self.__window_helper.build(profile)
	
	
class FormatMenu (Menu):
	""" 
	Abstract class for popup menus containing only LaTeX format commands 
	"""
	def __init__(self, window_helper):
		Menu.__init__(self)
		self.__window_helper = window_helper
	
	def insert_items(self, items):
		self.tooltips = Tooltips()
		for i in items:
			if not i:		## separator
				s = SeparatorMenuItem()
				s.show()
				self.append(s)
			else:			## menu item
				icon, label, source, tooltip = i	## unpack
				item = ImageMenuItem(label)
				if icon:
					image = Image()
					image.set_from_file("%s/pixmaps/%s.png" % (Installation().path, icon))
					item.set_image(image)
				item.show()
				item.connect("activate", self.__on_item_activated, source)
				self.tooltips.set_tip(item, u"⇨\t" + tooltip)
				self.append(item)
	
	def __on_item_activated(self, menu_item, source):
		helper = self.__window_helper.document_helper_manager.active_helper
		if helper.get_language() == LANGUAGE_LATEX:
			helper.language_helper.insert_source(source)
		else:
			debug(self, "__on_item_activated", "no LaTeX")


class JustifyMenu (FormatMenu):
	def __init__(self, instance):
		FormatMenu.__init__(self, instance)
		items = [
			("justify-center", _("Center"), {TEXT : ["\\begin{center}", "\end{center}"]}, "\\begin{center}"),
			("justify-left", _("Left align"), {TEXT : ["\\begin{flushleft}", "\end{flushleft}"]}, "\\begin{flushleft}"),
			("justify-right", _("Right align"), {TEXT : ["\\begin{flushright}", "\end{flushright}"]}, "\\begin{flushright}")
		]
		FormatMenu.insert_items(self, items)


# TODO: this doesn't work together with the SmartMenuToolButton because there's no GdkPixbuf
#
#from gtk import STOCK_JUSTIFY_CENTER
#
#class JustifyMenu (Menu):
#	def __init__(self, instance):
#		Menu.__init__(self)
#		self.instance = instance
#		
#		self.tooltips = Tooltips()
#		
#		item_center = ImageMenuItem(STOCK_JUSTIFY_CENTER)
#		item_center.show()
#		item_center.connect("activate", self.__on_item_activated, {TEXT : ["\\begin{center}", "\end{center}"]})
#		self.tooltips.set_tip(item_center, u"⇨\t" + "\\begin{center}")
#		self.append(item_center)
#	
#	def __on_item_activated(self, menu_item, source):
#		self.instance.get_active_wrapper().insert_source(source)


class FontStyleMenu (FormatMenu):
	def __init__(self, instance):
		FormatMenu.__init__(self, instance)
		items = [
			("bf", _("Bold"), {TEXT : ["\\textbf{", "}"], MATH : ["\\mathbf{", "}"]}, "\\textbf | \\mathbf"),
			("it", _("Italic"), {TEXT : ["\\textit{", "}"], MATH : ["\\mathit{", "}"]}, "\\textit | \\mathit"),
			("underline", _("Underline"), ["\\underline{", "}"], "\\underline"),
			("sc", _("Small Capitals"), {TEXT : ["\\textsc{", "}"]}, "\\textsc"),
			None,
			("rm", _("Roman"), {TEXT : ["\\textrm{", "}"], MATH : ["\\mathrm{", "}"]}, "\\textrm | \\mathrm"),
			("sf", _("Sans Serif"), {TEXT : ["\\textsf{", "}"], MATH : ["\\mathsf{", "}"]}, "\\textsf | \\mathsf"),
			("tt", _("Typewriter"), {TEXT : ["\\texttt{", "}"], MATH : ["\\mathtt{", "}"]}, "\\texttt | \\mathtt"),
			("bb", _("Blackboard Bold"), ConditionalSource({MATH : ["\\mathbb{", "}"]}, ["amsmath"]), "\\mathbb"),
			("cal", _("Caligraphy"), {MATH : ["\\mathcal{", "}"]}, "\\mathcal"),
			("frak", _("Fraktur"), ConditionalSource({MATH : ["\\mathfrak{", "}"]}, ["amsmath"]), "\\mathfrak")
		]
		FormatMenu.insert_items(self, items)


class FontSizeMenu (FormatMenu):
	def __init__(self, instance):
		FormatMenu.__init__(self, instance)
		items = [
			("decrease", _("Tiny"), {TEXT : ["\\begin{tiny}", "\\end{tiny}"]}, "\\begin{tiny}"),
			("decrease", _("Script Size"), {TEXT : ["\\begin{scriptsize}", "\\end{scriptsize}"]}, "\\begin{scriptsize}"),
			("decrease", _("Footnote Size"), {TEXT : ["\\begin{footnotesize}", "\\end{footnotesize}"]}, "\\begin{footnotesize}"),
			("decrease", _("Small"), {TEXT : ["\\begin{small}", "\\end{small}"]}, "\\begin{small}"),
			(None, _("Normal Size"), {TEXT : ["\\begin{normalsize}", "\\end{normalsize}"]}, "\\begin{normalsize}"),	## TODO: icon
			("increase", _("Large"), {TEXT : ["\\begin{large}", "\\end{large}"]}, "\\begin{large}"),
			("increase", _("Larger"), {TEXT : ["\\begin{Large}", "\\end{Large}"]}, "\\begin{Large}"),
			("increase", _("Very Large"), {TEXT : ["\\begin{LARGE}", "\\end{LARGE}"]}, "\\begin{LARGE}"),
			("increase", _("Huge"), {TEXT : ["\\begin{huge}", "\\end{huge}"]}, "\\begin{huge}"),
			("increase", _("Very Huge"), {TEXT : ["\\begin{Huge}", "\\end{Huge}"]}, "\\begin{Huge}")
		]
		FormatMenu.insert_items(self, items)


class StructureMenu (FormatMenu):
	def __init__(self, instance):
		FormatMenu.__init__(self, instance)
		items = [
			("part", _("Part"), {TEXT : ["\\part{", "}"]}, "\\part"),
			None,
			("chapter", _("Chapter"), {TEXT : ["\\chapter{", "}"]}, "\\chapter"),
			("section", _("Section"), {TEXT : ["\\section{", "}"]}, "\\section"),
			("subsection", _("Subsection"), {TEXT : ["\\subsection{", "}"]}, "\\subsection"),
			("subsubsection", _("Subsubsection"), {TEXT : ["\\subsubsection{", "}"]}, "\\subsubsection"),
			None,
			("paragraph", _("Paragraph"), {TEXT : ["\\paragraph{", "}"]}, "\\paragraph"),
			("paragraph", _("Subparagraph"), {TEXT : ["\\subparagraph{", "}"]}, "\\subparagraph")
		]
		FormatMenu.insert_items(self, items)


class MathMenu (FormatMenu):
	def __init__(self, instance):
		FormatMenu.__init__(self, instance)
		items = [
			("math", _("Mathematical Environment"), {TEXT : ["$ ", " $"]}, "$ $"),
			("displaymath", _("Centered Formula"), {TEXT : ["\\[ ", " \\]"]}, "\\[ \\]"),
			("equation", _("Numbered Equation"), {TEXT : ["\\begin{equation}\n\t", "\n\\end{equation}"]}, "\\begin{equation}"),
			("uneqnarray", _("Array of Equations"), ConditionalSource({TEXT : ["\\begin{align*}\n\t", "\n\\end{align*}"]}, ["amsmath"]), "\\begin{align*}"),
			("eqnarray", _("Numbered Array of Equations"), ConditionalSource({TEXT : ["\\begin{align}\n\t", "\n\\end{align}"]}, ["amsmath"]), "\\begin{align}")
		]
		FormatMenu.insert_items(self, items)


class AccentMenu (FormatMenu):
	def __init__(self, instance):
		FormatMenu.__init__(self, instance)
		items = [
			("dot", _("Dot"), {TEXT : ["\\.{", "}"], MATH : ["\\dot{", "}"]}, "\\. | \\dot"),
			("ddot", _("Double Dot"), {TEXT : ['\\"{', "}"], MATH : ["\\ddot{", "}"]}, '\\" | \\ddot'),
			("bar", _("Bar"), {MATH : ["\\bar{", "}"]}, "\\bar"),
			("tilde", _("Tilde"), {TEXT : ["\\~{", "}"], MATH : ["\\tilde{", "}"]}, "\\~ | \\tilde"),
			("vec", _("Vector"), {MATH : ["\\vec{", "}"]}, "\\vec"),
			("hat", _("Hat"), {TEXT : ["\\^{", "}"], MATH : ["\\hat{", "}"]}, "\\^ | \\hat"),
			("acute", _("Acute"), {TEXT : ["\\'{", "}"], MATH : ["\\acute{", "}"]}, "\\' | \\acute"),
			("grave", _("Grave"), {TEXT : ["\\`{", "}"], MATH : ["\\grave{", "}"]}, "\\` | \\grave"),
			("breve", _("Breve"), {MATH : ["\\breve{", "}"]}, "\\breve"),
			("check", _("Dot"), {MATH : ["\\check{", "}"]}, "\\check"),
		]
		FormatMenu.insert_items(self, items)


class SmartMenuToolButton(MenuToolButton):
	""" 
	A custom MenuToolButton that switches the icon and action of its button 
	to the last selected ones 
	"""
	active_item = None
	
	def __init__(self, label, menu, index=0):
		self.label = label
		item = menu.get_children()[index]
		self.image = Image()
		self.image.set_from_pixbuf(item.get_image().get_pixbuf())
		self.image.show()
		self.active_item = item
		
		MenuToolButton.__init__(self, self.image, label)
		self.button = self.get_children()[0].get_children()[0]
		
		self.__set_menu(menu)
	
	def __set_menu(self, menu):
		children = menu.get_children()
		for child in children:
			child.connect("activate", self.__on_item_activated)
		MenuToolButton.set_menu(self, menu)
		self.connect("clicked", self.__on_button_clicked)
		
	def __on_item_activated(self, item):
		""" called when a menu item is activated """
		## TODO: the button's label is lost here
		self.image.set_from_pixbuf(item.get_image().get_pixbuf())
		self.button.set_image(self.image)
		self.active_item = item

	def __on_button_clicked(self, button):
		""" called when the MenuToolButton is clicked """
		if self.active_item:
			self.active_item.emit("activate")
